# -*- coding: utf-8 -*-
# Generated by Django 1.11 on 2018-10-04 11:48
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('djangoldp', '0002_auto_20180913_1800'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='ldnotification',
            options={'permissions': (('view_todo', 'Read'), ('control_todo', 'Control'))},
        ),
    ]
